# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that defines the behavior of the play
button.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class Manager(object):
	"""
	This class defines the behavior of the play button.
	"""

	def __init__(self, trigger, player):
		"""
		Initialize object.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param trigger: Reference to the Trigger instance.
		@type trigger: A Trigger object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__init_attributes(trigger, player)
		self.__set_properties()
		self.__sig_id_1 = self.__button.connect("clicked", self.__clicked_cb, player)
		self.__sig_id_2 = player.connect("stream-error", self.__paused_cb)
		self.__sig_id_3 = player.connect("streamer-is-busy", self.__is_busy_cb)
		self.__sig_id_4 = player.connect("streamer-is-stopped", self.__paused_cb)
		self.__sig_id_5 = player.connect("playlist-error", self.__paused_cb)
		self.__sig_id_6 = player.connect("streamer-is-ready", self.__is_ready_cb)
		self.__sig_id_7 = player.connect("stream-message", self.__update_cb)
		self.__sig_id_8 = trigger.connect("destroy", self.__destroy_cb)

	def __init_attributes(self, trigger, player):
		"""
		Initialize data attributes.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param trigger: Reference to the Trigger instance.
		@type trigger: A Trigger object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__player = player
		self.__trigger = trigger
		self.__button = player.glade.get_widget("PlayButton")
		self.__image = player.glade.get_widget("PlayButtonImage")
		return

	def play(self):
		"""
		Play the radio station.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__button.activate()
		return

	def __set_properties(self):
		"""
		Set default properties.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		if self.__player.is_ready:
			if self.__player.is_busy:
				self.__set_stock_image("pause")
				self.__button.set_property("sensitive", False)
			elif self.__player.is_playing:
				self.__set_stock_image("pause")
				self.__button.set_property("sensitive", True)
			else:
				self.__set_stock_image()
				self.__button.set_property("sensitive", True)
		else:
			self.__set_stock_image()
			self.__button.set_property("sensitive", False)
		return

	def __set_stock_image(self, string="play"):
		"""
		Change icon on the play button.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param string: A string representing an icon to show.
		@type string: A String object.
		"""
		from gtk import STOCK_MEDIA_PLAY, STOCK_MEDIA_PAUSE, ICON_SIZE_BUTTON
		from operator import eq
		if eq(string, "play"):
			self.__image.clear()
			self.__image.set_from_stock(STOCK_MEDIA_PLAY, ICON_SIZE_BUTTON)
		else:
			self.__image.clear()
			self.__image.set_from_stock(STOCK_MEDIA_PAUSE, ICON_SIZE_BUTTON)
		return

	def __clicked_cb(self, *args):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__player.emit("toggle-play")
		return True

	def __paused_cb(self, *args):
		"""
		Handles callback when the radio player is paused.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__set_stock_image("play")
		self.__button.set_property("sensitive", True)
		return

	def __playing_cb(self, *args):
		"""
		Handles callback when the radio player starts streaming.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__set_stock_image("pause")
		self.__button.set_property("sensitive", True)
		return

	def __is_ready_cb(self, *args):
		"""
		Handles callback when the streamer is ready.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
	#	self.__set_stock_image()
	#	self.__button.set_property("sensitive", True)
		return

	def __is_busy_cb(self, *args):
		"""
		Handles callback when the streamer is busy.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__set_stock_image("pause")
		self.__button.set_property("sensitive", False)
		return

	def __update_cb(self, *args):
		"""
		Handles callback when the streamer updates metadata.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__button.set_property("sensitive", True)
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__sig_id_1, self.__button)
		disconnect_signal(self.__sig_id_2, self.__player)
		disconnect_signal(self.__sig_id_3, self.__player)
		disconnect_signal(self.__sig_id_4, self.__player)
		disconnect_signal(self.__sig_id_5, self.__player)
		disconnect_signal(self.__sig_id_6, self.__player)
		disconnect_signal(self.__sig_id_7, self.__player)
		disconnect_signal(self.__sig_id_8, self.__trigger)
		self.__button.destroy()
		delete_attributes(self)
		del self
		self = None
		return
